/**
 * @fileoverview Rule to flag declared but unused methods inside a react component
 * @author Nir
 */
"use strict";

function isReactComponent(node, context) {
  if (!node.superClass) {
    return false;
  }
  return /^(React\.)?(Pure)?Component$/.test(context.getSourceCode().getText(node.superClass));
}

module.exports = {
  meta: {
    docs: {
      description: "Disallow unused methods in react components",
      category: "Methods",
      recommended: false
    },
    fixable: null,  // or "code" or "whitespace"
    schema: [
      // fill in your schema
    ]
  },

  create: function (context) {
    const ignoreMethods = [
      'componentWillMount',
      'componentDidMount',
      'componentWillUpdate',
      'componentDidUpdate',
      'render',
      'propTypes',
      'componentWillReceiveProps',
      'shouldComponentUpdate',
      'componentWillUnmount',
      ]
    let methods = {};

    function addMethod(methodName, node) {
      if (methods[methodName]) {
        methods[methodName].node = node;
      } else {
        methods[methodName] = { node, methodName, isUsed: false };
      }
    }
    // variables should be defined here

    //----------------------------------------------------------------------
    // Helpers
    //----------------------------------------------------------------------

    // any helper functions should go here or else delete this section

    //----------------------------------------------------------------------
    // Public
    //----------------------------------------------------------------------
    return {

      // 'ClassDeclaration[superClass.name="Component"]': (node) => {
      //   console.log('nirrrrrrrrrrrrrrrrrrrrrrrrr');
      // },
      'MethodDefinition': (node) => {
        addMethod(node.key.name, node);
      },
      'ClassProperty': (node) => {
        addMethod(node.key.name, node);
      },
      'MemberExpression': (node) => {
        if (node.object.type === 'ThisExpression') {
          const methodName = node.property.name;
          if (!methods[methodName]) {
            addMethod(methodName);
          }
          methods[methodName].isUsed = true;
        }
      },
      'ClassDeclaration:exit': (node) => {
        if (isReactComponent(node, context)) {
          const unUsedMethods = Object.keys(methods)
            .filter(key => !methods[key].isUsed)
            .filter(key => !ignoreMethods.includes(methods[key].methodName))
            .map(key => methods[key]);
          unUsedMethods.forEach(method => context.report({
            node: method.node,
            message: 'unused class method {{methodName}}',
            data: {
              methodName: method.methodName
            }
          }));
        }
        methods = {};
      }
    };
  }
};
